/*
	
	CallerIDd.cpp v0.8.1
	
	Copyright 1997 Jeremy Rand

This file contains the code which builds the CallerID
daemon application.
	
*/

#include "CallerIDd.h"

main()
{

	// Pretty standard main() for a BeOS app
	
	CIDApplication *myApplication;
	
	myApplication = new CIDApplication();
	myApplication->Run();
	
	delete(myApplication);
	return(0);
	}


CIDApplication::CIDApplication()
		  		  : BApplication(CallerIDdType)
{
	// On construction, just read in any existing CallerID settings file
	// Create a default one if it doesn't exist.
	
	char buffer[256];
	FILE *fp;
	
	if ((fp=fopen("/boot/system/settings/CallerID", "r")) == NULL) {
		fp=fopen("/boot/system/settings/CallerID", "w");
		fprintf(fp, "serial2\n%ld\n%ld\n%ld\n%ld\n\\rATZ\\r\n", B_57600_BPS, B_DATA_BITS_8,
			B_STOP_BITS_1, B_NO_PARITY);
		fclose(fp);
		fp=fopen("/boot/system/settings/CallerID", "r");
		}
	
	fgets(buffer, 255, fp);
	buffer[strlen(buffer)-1]=0;
	strcpy(theSettings.serialport,buffer);
	
	fgets(buffer, 255, fp);
    theSettings.theDataRate=data_rate(strtol(buffer, NULL, 10));
    
    fgets(buffer, 255, fp);
    theSettings.theDataBits=data_bits(strtol(buffer, NULL, 10));
    
    fgets(buffer, 255, fp);
    theSettings.theStopBits=stop_bits(strtol(buffer, NULL, 10));
    
    fgets(buffer, 255, fp);
    theSettings.theParityMode=parity_mode(strtol(buffer, NULL, 10));
    
    fgets(buffer, 255, fp);
	buffer[strlen(buffer)-1]=0;
	theSettings.initstring=ParseInit(buffer);
	
	theSettingsWindow=new BMessenger();
	}


char *CIDApplication::ParseInit(char *buffer)
{
	// This function turns \r and \n's from the input buffer into carriage returns.
	// It allocates the string for its result.
	
	char *result, *i;
	
	result=new char[strlen(buffer) + 5];
	result[0]='\r';				// Always start an init string with a carriage return
		
	for(i=result+1; *buffer != 0; buffer++, i++) {
		// This parses the \r and \n's into actual eol characters
		if ( *buffer == '\\' ) {
			buffer++;
			switch (*buffer) {
				case 'r':
				case 'n':
					*i = '\r';
					break;
				case 0:
					*i = '\\';
					buffer--;
					break;
				default:
					*i = *buffer;
				}
			}
		else
			*i = *buffer;
		}
	
	*i='\r';				// Always end an init string with a carriage return
	i++;
	*i=0;		// Don't forget to NULL terminate
	return(result);
	}


void CIDApplication::MessageReceived(BMessage *msg)
{
	// Just watching incoming BMessages
	
	long result;
	
	switch (msg->what) {
		case MSG_STARTTHREAD:
		case MSG_STOPTHREAD:
				
			msg->FindMessenger("return", theSettingsWindow);
					// This is a BMessager to talk to the preferences window
											
			if (ReaderRunning) {	// Either case, if it is running, stop it
				ReaderRunning=FALSE;
				kill(reader, SIGUSR1);
				wait_for_thread(reader, &result);
				}
		
			if (msg->what == MSG_STOPTHREAD)
				break;				// If this is a stop, we are done
				
			strcpy(theSettings.serialport,(char *)msg->FindString("serialport"));
			theSettings.theDataRate=data_rate(msg->FindInt32("portspeed"));
			theSettings.theDataBits=data_bits(msg->FindInt32("databits"));
			theSettings.theStopBits=stop_bits(msg->FindInt32("stopbits"));
			theSettings.theParityMode=parity_mode(msg->FindInt32("parity"));
			if (theSettings.initstring != NULL)
				delete[] theSettings.initstring;
			theSettings.initstring=ParseInit((char *)msg->FindString("initstring"));
								// Store the settings from the message
								
			if (!IsLaunching()) {	// If we are launching, we start the thread in the ReadyToRun func
				ReaderRunning=TRUE;
				reader = spawn_thread(reader_thread, "reader", B_NORMAL_PRIORITY, (void *) &theSettings);
				resume_thread(reader);
				}
				
			break;
		
		case MSG_READERDEAD:
			if (theSettingsWindow->IsValid())
				theSettingsWindow->SendMessage(MSG_ISSTOPPED);
			break;
		
		case MSG_READERRUNNING:
			if (theSettingsWindow->IsValid())
				theSettingsWindow->SendMessage(MSG_ISRUNNING);
			break;
		
		case MSG_HELLO:				// Preferences sends this to give us a BMessenger
				
			if (msg->FindMessenger("return", theSettingsWindow) == B_NO_ERROR) {
				if (ReaderRunning)		// Tell them if the reader is running or not
					theSettingsWindow->SendMessage(MSG_ISRUNNING);
				else
					theSettingsWindow->SendMessage(MSG_ISSTOPPED);
				}
			else {
				BAlert *anAlert = new BAlert("",
					"Settings sent a bad Hello message!",
					"OK");
				anAlert->Go();						// Display Alert if we can't start it
				}
			break;
			
		default:								// Handle all other BMessages as usual
			inherited::MessageReceived(msg);
		}
	}


void CIDApplication::ReadyToRun(void)
{
	// Start up the thread once environment is set up
	
	if (!ReaderRunning) {
		ReaderRunning=TRUE;
		reader = spawn_thread(reader_thread, "reader", B_NORMAL_PRIORITY, (void *) &theSettings);
		resume_thread(reader);
		}
	}


bool CIDApplication::QuitRequested()
{	
	long result;
	
	if (ReaderRunning) {		// If reader is running, stop it
		ReaderRunning=FALSE;
		kill(reader, SIGUSR1);
		wait_for_thread(reader, &result);
		}
	else if (theSettingsWindow->IsValid())
		theSettingsWindow->SendMessage(MSG_ISQUIT);	// Make sure preferences knows its stopped
	
	delete theSettingsWindow;
	
	return(inherited::QuitRequested());
	}