/*

	Reader.h v0.8.1
    
	Copyright 1997 Jeremy Rand
    
*/

#ifndef READER_H
#define READER_H

#include <kernel/OS.h>
#include <interface/Alert.h>
#include <app/Message.h>
#include <app/Messenger.h>
#include <app/Roster.h>
#include <string.h>
#include <ctype.h>
#include <signal.h>
#include <device/SerialPort.h>
#include "../CallerIDCommon.h"

const int CND_FIRST_TOKEN=0;
const int CND_EQUALS=0;
const int CND_RING=1;
const int CND_OTIME=3;
const int CND_ODATE=4;
const int CND_ONUMBER=5;
const int CND_ONAME=6;
const int CND_ZTIME=2;
const int CND_ZNUMBER=7;
const int CND_ZNAME=8;
const int CND_ZNONUMBER=9;
const int CND_ZNONAME=10;
const int NUM_TOKENS=CND_ZNONAME+1;
const long BUFFERSIZE=512;

extern int ReaderRunning;
extern BMessenger *theSettingsWindow;

struct SerialSet {
	char serialport[32];
	data_rate theDataRate;
	data_bits theDataBits;
	stop_bits theStopBits;
	parity_mode theParityMode;
	char *initstring;
	};


long reader_thread(void *);
void ReaderDie(long);
void handler(int);


class CNDReader {

protected:

	char buffer[BUFFERSIZE];
	char *cursor, *end, *calltime, *callname, *callnum;
	BSerialPort *modem;
	char *CND_Strings[NUM_TOKENS];
	void SkipWS(void);
	void NextWord(void);
	void GetMore(void);
	int GetNextToken(void);
	char *GetRestLine(void);
		
public:
	class Normal_Quit { };			// exception class
	class CND_Error { };			// exception class
	void GetNextCall(void);
	char *GetDateTime(void);
	char *GetCallerName(void);
	char *GetCallerNum(void);
	CNDReader(const char *portname, const char *initstring, data_rate CND_data_rate,
		data_bits CND_data_bits, parity_mode CND_parity_mode, stop_bits CND_stop_bits);
	~CNDReader();
	};

#endif